import pandas as pd
import numpy as np
from cassandra.data.trasformations.trasformations import saturation
from scipy.stats import dweibull
import matplotlib.pyplot as plt
import plotly.express as px
import plotly.graph_objects as go


# def saturation_to_dataset(df, df_adstock_saturation):
#     df_saturation = df.copy()
#
#     for index, row in df.iterrows():
#         for m in list(df_adstock_saturation['canale']):
#             df_saturation.at[index, m] = saturation(df_saturation.at[index, m], df_adstock_saturation.loc[
#                 df_adstock_saturation['canale'] == m, 'saturation'].iloc[0])
#
#     return df_saturation


def response_regression_to_dataset(df, name_date_column, name_target_colum, name_prediction_column, coef_dict):
    df['intercept'] = 1
    new_dict = {}

    for key, value in coef_dict.items():
        new_dict[key] = df[key] * coef_dict[key]

    response_df = df[[name_date_column, name_target_colum, name_prediction_column]].join(pd.DataFrame(new_dict))
    return response_df


def decomposition_to_dataset(df, coef_dict, medias, spend_df, name_date_column, name_target_colum,
                             name_prediction_column, df_adstock_saturation):
    df_all_decomp = response_regression_to_dataset(df, name_date_column, name_target_colum, name_prediction_column,
                                                   coef_dict)

    coef_df = pd.DataFrame(list(coef_dict.items()))
    coef_df.rename(columns={coef_df.columns[0]: 'canale', coef_df.columns[1]: 'coef'}, inplace=True)

    decomp_agg_df = pd.DataFrame(df_all_decomp[coef_dict.keys()].sum(axis=0))
    decomp_agg_df.reset_index(inplace=True)
    decomp_agg_df.rename(columns={decomp_agg_df.columns[0]: 'canale', decomp_agg_df.columns[1]: 'xDecompAgg'},
                         inplace=True)

    decomp_mean_df = pd.DataFrame(df_all_decomp[coef_dict.keys()][df_all_decomp[coef_dict.keys()] > 0].mean(axis=0))
    decomp_mean_df.reset_index(inplace=True)
    decomp_mean_df.rename(columns={decomp_mean_df.columns[0]: 'canale', decomp_mean_df.columns[1]: 'xDecompMeanNon0'},
                          inplace=True)

    decomp_df = pd.merge(left=decomp_agg_df, right=decomp_mean_df, left_on='canale', right_on='canale')

    response_decomp_df = pd.merge(left=decomp_df, right=coef_df, left_on='canale', right_on='canale')

    response_decomp_df['xDecompPerc'] = response_decomp_df['xDecompAgg'] / response_decomp_df['xDecompAgg'].sum()
    response_decomp_df['xDecompMeanNon0Perc'] = response_decomp_df['xDecompMeanNon0'] / response_decomp_df[
        'xDecompMeanNon0'].sum()

    df_aggregated = pd.merge(how='left', left=response_decomp_df, right=spend_df, left_on='canale',
                             right_on='canale')

    df_aggregated = pd.merge(how='left', left=df_aggregated, right=df_adstock_saturation, left_on='canale',
                             right_on='canale')

    xDecompPercSum = 0

    for index, row in df_aggregated.iterrows():
        if row['canale'] in medias:
            xDecompPercSum = xDecompPercSum + row['xDecompPerc']

    for index, row in df_aggregated.iterrows():
        if row['canale'] in medias:
            df_aggregated.at[index, 'effect_share'] = (df_aggregated.at[index, 'xDecompPerc'] / xDecompPercSum) * 100
            df_aggregated.at[index, 'roi'] = (df_aggregated.at[index, 'xDecompAgg'] / df_aggregated.at[
                index, 'spesa']) * 100
            df_aggregated.at[index, 'cpa'] = df_aggregated.at[index, 'spesa'] / df_aggregated.at[
                index, 'xDecompAgg']
            df_aggregated.at[index, 'response_media'] = df_aggregated.at[index, 'coef'] * saturation(df_aggregated.at[
                                                                                                         index, 'xDecompMeanNon0'],
                                                                                                     df_aggregated.at[
                                                                                                         index, 'saturation'])
            df_aggregated.at[index, 'roi_medio'] = (df_aggregated.at[index, 'response_media'] / df_aggregated.at[
                index, 'spesa_media']) * 100

    # TODO
    # df_aggregated = df_aggregated[['canale', 'xDecompAgg', 'xDecompMeanNon0', 'xDecompPerc', 'xDecompMeanNon0Perc', 'spesa', 'spesa_media',
    # 'spend_share', 'effect_share', 'spesa_totale', 'roi', 'roi_medio', 'cpa', 'response_media']]

    return df_aggregated, df_all_decomp


def adstock_saturation_to_dataset(nevergrad_dict, adstock_type, weibull_type='pdf', max_y=1, max_x=100, graph=True):
    if adstock_type == 'weibull':
        result_df = pd.DataFrame()
        adstock_columns = [col.replace('_shape', '') for col in nevergrad_dict.keys() if 'shape' in col]

        for m in adstock_columns:
            shape_name = m + '_shape'
            scale_name = m + '_scale'
            shape = nevergrad_dict[shape_name]
            scale = nevergrad_dict[scale_name]

            x_array = np.array([1])
            x = np.repeat(x_array, 100, axis=0)  # see to del
            range_x = [x / 100 for x in range(1, 101)]

            if weibull_type == 'pdf':
                y = dweibull.pdf(range_x, shape, scale=scale)
            else:
                y = dweibull.cdf(range_x, shape, scale=scale)

            data = plt.plot(range(1, 101), y)[0].get_data()

            df_media = pd.DataFrame(data).T

            df_media.rename(columns={df_media.columns[0]: 'x', df_media.columns[1]: 'y'},
                            inplace=True)

            for index, row in df_media.iterrows():
                df_media.at[index, 'y'] = ((df_media.at[index, 'y'] - df_media['y'].min()) / (
                        df_media['y'].max() - df_media['y'].min())) * (max_y - 0) + 0
                df_media.at[index, 'x'] = ((df_media.at[index, 'x'] - df_media['x'].min()) / (
                        df_media['x'].max() - df_media['x'].min())) * (max_x - 0) + 0
            if graph == True:
                title_graph = 'Adstock weibull ' + m
                fig2 = px.line(x=df_media['x'], y=df_media['y'], title=title_graph)
                fig2.show()

            df_media['canale'] = m
            result_df = pd.concat([result_df, df_media])

        shapes = [col for col in nevergrad_dict.keys() if 'shape' in col]
        adstocked_shapes = {key: nevergrad_dict[key] for key in shapes}
        scales = [col for col in nevergrad_dict.keys() if 'scale' in col]
        adstocked_scales = {key: nevergrad_dict[key] for key in scales}
        betas = [col for col in nevergrad_dict.keys() if 'beta' in col]
        saturationed = {key: nevergrad_dict[key] for key in betas}

        adstock_shapes_df = pd.DataFrame(list(adstocked_shapes.items()))
        adstock_shapes_df.rename(
            columns={adstock_shapes_df.columns[0]: 'canale', adstock_shapes_df.columns[1]: 'adstock_shape'},
            inplace=True)
        adstock_shapes_df['canale'] = adstock_shapes_df.canale.str.replace('_shape', '')

        adstock_scales_df = pd.DataFrame(list(adstocked_scales.items()))
        adstock_scales_df.rename(
            columns={adstock_scales_df.columns[0]: 'canale', adstock_scales_df.columns[1]: 'adstock_scale'},
            inplace=True)
        adstock_scales_df['canale'] = adstock_scales_df.canale.str.replace('_scale', '')

        adstock_df = pd.merge(left=adstock_shapes_df, right=adstock_scales_df, left_on='canale',
                              right_on='canale')

        saturation_df = pd.DataFrame(list(saturationed.items()))
        saturation_df.rename(columns={saturation_df.columns[0]: 'canale', saturation_df.columns[1]: 'saturation'},
                             inplace=True)
        saturation_df['canale'] = saturation_df.canale.str.replace('_beta', '')

        df_adstock_saturation = pd.merge(left=adstock_df, right=saturation_df, left_on='canale',
                                         right_on='canale')

    elif adstock_type == 'geometric':

        thetas = [col for col in nevergrad_dict.keys() if 'theta' in col]
        adstocked = {key: nevergrad_dict[key] for key in thetas}
        betas = [col for col in nevergrad_dict.keys() if 'beta' in col]
        saturationed = {key: nevergrad_dict[key] for key in betas}

        adstock_df = pd.DataFrame(list(adstocked.items()))
        adstock_df.rename(columns={adstock_df.columns[0]: 'canale', adstock_df.columns[1]: 'adstock'}, inplace=True)
        adstock_df['canale'] = adstock_df.canale.str.replace('_theta', '')

        saturation_df = pd.DataFrame(list(saturationed.items()))
        saturation_df.rename(columns={saturation_df.columns[0]: 'canale', saturation_df.columns[1]: 'saturation'},
                             inplace=True)
        saturation_df['canale'] = saturation_df.canale.str.replace('_beta', '')

        df_adstock_saturation = pd.merge(left=adstock_df, right=saturation_df, left_on='canale',
                                         right_on='canale')

        if graph == True:
            fig = go.Figure(go.Bar(
                x=round(df_adstock_saturation['adstock'] * 100, 2),
                y=df_adstock_saturation['canale'],
                orientation='h'),
                layout=dict(height=700, width=1000, xaxis_title='adstock geometric (%)', yaxis_title='canale',
                            title='Geometric Adstock', legend_title='Legend')
            )

            fig.show()

    if adstock_type == 'weibull':
        return df_adstock_saturation, result_df
    else:
        return df_adstock_saturation


def spend_to_dataset(df, medias):
    total_spend_dict = {key: df[key].sum() for key in medias}
    total_spend_df = pd.DataFrame(list(total_spend_dict.items()))
    total_spend_df.rename(columns={total_spend_df.columns[0]: 'canale', total_spend_df.columns[1]: 'spesa'},
                          inplace=True)

    mean_spend_dict = {key: df[key].mean() for key in medias}
    mean_spend_df = pd.DataFrame(list(mean_spend_dict.items()))
    mean_spend_df.rename(columns={mean_spend_df.columns[0]: 'canale', mean_spend_df.columns[1]: 'spesa_media'},
                         inplace=True)

    result_df = pd.merge(left=total_spend_df, right=mean_spend_df, left_on='canale',
                         right_on='canale')

    result_df['spesa_totale'] = sum(total_spend_dict.values())
    result_df['spend_share'] = (result_df['spesa'] * 100) / result_df['spesa_totale']

    return result_df


def saturation_to_dataset(df, df_adstock_saturation, coeffs, features):
    df_saturation = df.copy()

    for index, row in df_saturation.iterrows():
        for m in list(df_adstock_saturation['canale']):
            df_saturation.at[index, m] = saturation(df_saturation.at[index, m], df_adstock_saturation.loc[
                df_adstock_saturation['canale'] == m, 'saturation'].iloc[0])

    coef_array = coeffs.copy()
    new_dict = {}

    for x in range(len(coef_array) - 1):
        new_dict[features[x]] = df_saturation[features[x]] * coef_array[x]

    df_saturation_response = pd.DataFrame(new_dict)
    return df_saturation_response


# def decomposition_to_dataset(df, coef_dict, features, spend_df, medias, name_date_column, name_target_colum,
#                              name_prediction_column):
#     columns = features
#     coef_array = list(coef_dict.values())
#     result = df[columns].copy()
#
#     result['intercept'] = coef_array[-1]
#
#     coef_dict = {}
#
#     for x in range(len(coef_array) - 1):
#         coef_dict[columns[x]] = coef_array[x]
#
#     coef_dict['intercept'] = coef_array[-1]
#
#     df_all_decomp = response_regression_to_dataset(df, name_date_column, name_target_colum, name_prediction_column,
#                                                    coef_dict, features)
#
#     coef_df = pd.DataFrame(list(coef_dict.items()))
#     coef_df.rename(columns={coef_df.columns[0]: 'canale', coef_df.columns[1]: 'coef'}, inplace=True)
#
#     decomp_df = pd.DataFrame(df_all_decomp[result.keys()].sum(axis=0))
#     decomp_df.reset_index(inplace=True)
#     decomp_df.rename(columns={decomp_df.columns[0]: 'canale', decomp_df.columns[1]: 'xDecompAgg'},
#                      inplace=True)
#
#     response_decomp_df = pd.merge(left=decomp_df, right=coef_df, left_on='canale', right_on='canale')
#
#     response_decomp_df['xDecompPerc'] = response_decomp_df['xDecompAgg'] / response_decomp_df['xDecompAgg'].sum()
#
#     df_aggregated = pd.merge(how='left', left=response_decomp_df, right=spend_df, left_on='canale',
#                              right_on='canale')
#
#     xDecompPercSum = 0
#
#     for index, row in df_aggregated.iterrows():
#         if row['canale'] in medias:
#             xDecompPercSum = xDecompPercSum + row['xDecompPerc']
#
#     for index, row in df_aggregated.iterrows():
#         if row['canale'] in medias:
#             df_aggregated.at[index, 'effect_share'] = df_aggregated.at[index, 'xDecompPerc'] / xDecompPercSum
#             df_aggregated.at[index, 'roi'] = df_aggregated.at[index, 'xDecompAgg'] / df_aggregated.at[
#                 index, 'spesa_totale']
#
#     return df_aggregated, df_all_decomp



import plotly.express as px
import plotly.graph_objects as go
from plotly.subplots import make_subplots


# We can then compare the Sales distribution with media spent distribution to find correlations
# Define an array with all the media spent names
# media_spent = ['g_discovery_cost', 'g_display_cost', 'g_video_cost', 'g_search_brand_cost',
# 'g_search_no_brand_cost', 'b_audience_cost', 'b_search_cost', 'pinterest_cost', 'fb_cost']
def show_target_vs_media_spent_graph(df, name_data_column, name_target_column, medias_spent, title_y2='Medias Spent'):
    subfig = make_subplots(specs=[[{"secondary_y": True}]])

    fig = px.line(df, x=name_data_column, y=name_target_column)

    fig2 = px.line(df, x=name_data_column, y=medias_spent)
    fig2.update_traces(yaxis="y2")

    subfig.add_traces(fig.data + fig2.data)
    subfig.layout.xaxis.title = name_data_column
    subfig.layout.yaxis.title = name_target_column
    subfig.layout.yaxis2.title = title_y2
    subfig.for_each_trace(lambda t: t.update(line=dict(color=t.marker.color)))
    subfig.show()


# We may want to compare Predicted vs. Actual sales
def show_prediction_vs_actual_graph(df, name_data_column, name_target_column, name_prediction_column):
    fig = px.line(df, x=name_data_column, y=[name_target_column, name_prediction_column])
    fig.show()


# We may want to compare Predicted vs. Actual sales with error
def show_prediction_vs_actual_graph_with_error(df, target_column, prediction_column, name_data_column,
                                               name_target_column, name_model=''):
    y_error = (prediction_column - target_column) / target_column * 100

    title_accuracy_graph = name_model + ' Model: Actual vs Predicted'

    accuracy_chart = go.Figure(
        layout=dict(height=700, width=1000, xaxis_title=name_data_column, yaxis_title=name_target_column,
                    title=title_accuracy_graph, legend_title='Legend'))
    accuracy_chart.add_trace(go.Scatter(name="Actual", x=df[name_data_column], y=target_column))
    accuracy_chart.add_trace(go.Scatter(name="Predicted", x=df[name_data_column], y=prediction_column))
    accuracy_chart.show()

    title_error_graph = name_model + ' Model Error of Actual vs Predicted'

    error_chart = go.Figure(
        layout=dict(xaxis_title=name_data_column, yaxis_title=name_target_column + ' error', title=title_error_graph,
                    legend_title='Legend'))
    error_chart.add_trace(go.Scatter(name="Error", x=df[name_data_column], y=y_error))
    error_chart.show()


def show_metric_trend(metric_array, name_metric='mape'):
    fig = go.Figure(
        layout=dict(xaxis_title='Iterations', yaxis_title=name_metric, title=name_metric + ' Trend',
                    legend_title='Legend'))
    fig.add_trace(go.Scatter(y=metric_array))
    fig.show()



from IPython.core.display import display
from statsmodels.stats.outliers_influence import variance_inflation_factor
import pandas as pd


def check_multicollinearity(X):
    df = pd.DataFrame()
    df['feature'] = X.columns
    # calculating VIF for each feature
    df['VIF'] = [variance_inflation_factor(X.values, i) for i in range(len(X.columns))]
    display(df)



from IPython.display import Markdown, display
import seaborn as sns
import matplotlib.pyplot as plt
import plotly.express as px


def show_trendline_curves(features, df, name_target_column):
    trendline_df = df

    for f in features:
        # For each media remove all the 0 spend values as Log(0) can't be calculated
        trendline_df[f] = trendline_df[f][trendline_df[f] > 0]
        # saturation_df = saturation_df[(saturation_df[[media]] != 0).all(axis=1)]

        # Create a Scatter Plot
        fig = px.scatter(
            # Our temporary DF with no 0 values
            trendline_df,
            # Features name
            x=f,
            # Our output variable (Sales or Conversion)
            y=name_target_column,
            # Specify the type of trendline you want to show, use OLS if not sure
            trendline="ols",
            # Define the trendline as logarithmic, this will return curved functions
            trendline_options=dict(log_x=True)
        )
        # Print Figure
        fig.show()


def show_relationship_between_variables(df, plot_for_line=5, show_only_one=''):
    # Iterative pair-plotting
    # Just set the @plot_for_line and eventually change DF name and it will do the rest

    # df already in scope as the pandas dataframe with data
    hue = 'label'

    # How many plots per row you want to display
    all_vars = list(df.columns.symmetric_difference([]))

    if show_only_one:
        display(Markdown(f"### {show_only_one}"))

        for var in all_vars:
            display(Markdown(f"**{var} vs {show_only_one}**"))
            sns.pairplot(df, x_vars=[var], y_vars=show_only_one, palette='bright')
            plt.show()
            plt.close()

    else:
        for var in all_vars:
            rest_vars = list(all_vars)
            rest_vars.remove(var)
            display(Markdown(f"### {var}"))

            while rest_vars:
                line_vars = rest_vars[:plot_for_line]
                del rest_vars[:plot_for_line]
                line_var_names = ", ".join(line_vars)
                display(Markdown(f"**{var} vs {line_var_names}**"))
                sns.pairplot(df, x_vars=line_vars, y_vars=[var], palette='bright', )
                plt.show()
                plt.close()



import pandas as pd
import numpy as np
import plotly.express as px
from cassandra.model.modelEvaluation.evaluation import saturation_to_dataset
from sklearn import metrics
import plotly.graph_objects as go
import math


def show_nrmse(y_actual, y_pred, verbose=False):
    value = np.sqrt(np.mean((y_actual - y_pred) ** 2)) / (max(y_actual) - min(y_actual))
    passed = "✔️" if value < 0.15 else "❌"
    if verbose:
        return value, passed
    else:
        return value


def show_mape(y_actual, y_pred, verbose=False):
    # mean absolute percentage error
    x = np.abs((y_actual - y_pred) / y_actual)
    value = np.mean(x[~np.isinf(x)])
    passed = "✔️" if value < 0.15 else "❌"
    if verbose:
        return value, passed
    else:
        return value


def show_rsquared(y_actual, y_pred, verbose=False):
    # r squared
    value = round(metrics.r2_score(y_actual, y_pred), 3)
    passed = "✔️" if value > 0.8 else "❌"
    if verbose:
        return value, passed
    else:
        return value


def show_nrmse_old(y_actual, y_pred, verbose=False):
    # normalized root mean square error
    value = round(np.sqrt(metrics.mean_squared_error(y_actual, y_pred)) / np.mean(y_actual), 3)
    passed = "✔️" if value < 0.15 else "❌"
    if verbose:
        return value, passed
    else:
        return value


def show_mape_old(y_actual, y_pred, verbose=False):
    # mean absolute percentage error
    value = round(metrics.mean_absolute_error(y_actual, y_pred) / np.mean(y_actual), 3)
    passed = "✔️" if value < 0.15 else "❌"
    if verbose:
        return value, passed
    else:
        return value


# TODO
def show_rssd(df_aggregated, medias, verbose=False):
    # mean absolute percentage error
    df_result = pd.DataFrame()

    for index, row in df_aggregated.iterrows():
        if row['canale'] in medias:
            df_result.at[index, 'share'] = (df_aggregated.at[index, 'effect_share'] - df_aggregated.at[
                index, 'spend_share']) ** 2

    value = math.sqrt(df_result.sum())

    passed = "✔️" if value < 0.15 else "❌"
    if verbose:
        return value, passed
    else:
        return value


def show_coefficients(features, model, name_model, graph=True):
    # Given model = LinearRegression() model already executed

    # Create an array of the variables you want to check coeffs
    # features = ['g_display_cost', 'g_shopping_cost', 'g_video_cost', 'g_search_brand_cost', 'g_search_no_brand_cost',
    #             'fb_cost', 'pinterest_cost', 'b_audience_cost', 'b_search_cost', 'avg_price',
    #             'solostove_organic_traffic', 'solostove_paid_traffic', 'trend_smokeless_fire_pit']

    coeffs = model.coef_.copy()
    new_features = features.copy()

    if model.intercept_:
        coeffs = np.append(coeffs, model.intercept_)
        new_features = np.append(new_features, 'intercept')

    roas = pd.DataFrame(data=coeffs, index=new_features, columns=['contribution'])
    title_graph = name_model + " Model Coefficients graph"
    if graph == True:

        fig = go.Figure(
            data=[go.Bar(x=roas.index, y=roas['contribution'])],
            layout=go.Layout(
                title=go.layout.Title(text=title_graph)
            )
        )

        fig.show()
        return coeffs
    else:
        return coeffs


def show_saturation_curves(df, df_adstock_saturation, coeffs, features, medias):
    df_saturation = saturation_to_dataset(df, df_adstock_saturation, coeffs, features)

    for m in medias:
        s_df = pd.merge(df[m], df_saturation[m], left_index=True, right_index=True)

        # For each media remove all the 0 spend values as Log(0) can't be calculated
        s_df[m + '_x'] = s_df[m + '_x'][s_df[m + '_x'] != 0]
        s_df[m + '_y'] = s_df[m + '_y'][s_df[m + '_y'] != 0]

        # Create a Scatter Plot
        fig = px.line(
            # Spends Media channel's name
            x=s_df[m + '_x'].sort_values(),
            # Response Media channel's name
            y=s_df[m + '_y'].sort_values()
        )
        fig.update_layout(
            xaxis_title=m + ' spend',
            yaxis_title=m + ' response')
        # Print Figure
        fig.show()

    return df_saturation


def show_effect_share_vs_spend_share(df_aggregated, show_cpa=True):
    df_spend_var = df_aggregated.copy().dropna()
    fig = go.Figure()

    fig.add_bar(x=df_spend_var['canale'], y=round(df_spend_var['spend_share'], 2), name='spend_share',
                marker=dict(color="Purple"))
    fig.add_bar(x=df_spend_var['canale'], y=round(df_spend_var['effect_share'], 2), name='effect_share',
                marker=dict(color="MediumPurple"))

    if show_cpa:
        fig.add_scatter(x=df_spend_var['canale'], y=round(df_aggregated['cpa'], 2), mode="markers",
                        name='cpa',
                        marker=dict(size=20, color="LightSeaGreen"))
    else:
        fig.add_scatter(x=df_spend_var['canale'], y=round(df_spend_var['roi'], 2), mode="markers", name='roi',
                        marker=dict(size=20, color="LightSeaGreen"))

    fig.show()


def show_all_decomp_response(df_all_decomp, name_date_column, features, show_target=False, target='conversions',
                             show_target_prediction=False, name_prediction_column='prediction', nbins=None, bargap=0.2):
    if nbins:
        nbins = len(df_all_decomp)
    if show_target:
        features.append(target)
    if show_target_prediction:
        features.append(name_prediction_column)

    fig = px.histogram(df_all_decomp, x=name_date_column, y=features, nbins=nbins)
    fig.update_layout(bargap=bargap)
    fig.show()


def show_seasonality(df, seasonality_columns, name_date_column):
    df_seasonality = df.copy()[seasonality_columns]

    for s in seasonality_columns:
        if s == 'trend':
            fig = px.line(
                x=pd.to_datetime(df[name_date_column]),
                y=df['trend'])
            fig.update_layout(
                xaxis_title='Date',
                yaxis_title='Trend')
            fig.show()

        if s == 'holidays':
            fig = px.line(
                x=pd.to_datetime(df[name_date_column]),
                y=df['holidays'])
            fig.update_layout(
                xaxis_title='Date',
                yaxis_title='Holidays')
            fig.show()

        if s == 'yearly':
            fig = px.line(
                x=pd.to_datetime(df[name_date_column]).dt.dayofyear,
                y=df['yearly'])
            fig.update_layout(
                xaxis_title='Day of year',
                yaxis_title='Yearly')
            fig.show()

        if s == 'monthly':
            fig = px.line(
                x=pd.to_datetime(df[name_date_column]).dt.day,
                y=df['monthly'])
            fig.update_layout(
                xaxis_title='Day of month',
                yaxis_title='Montly')
            fig.show()

        if s == 'weekly':
            fig = px.line(
                x=pd.to_datetime(df[name_date_column]).dt.day_name(),
                y=df['weekly'])
            fig.update_layout(
                xaxis_title='Days',
                yaxis_title='Weekly')
            fig.show()

        if s == 'daily':
            fig = px.line(
                x=pd.to_datetime(df[name_date_column]).dt.time,
                y=df['daily'])
            fig.update_layout(
                xaxis_title='Hour of Day',
                yaxis_title='Daily')
            fig.show()

    return df_seasonality

# def show_geometric_adstock(df_adstock_saturation):
#     fig = go.Figure(go.Bar(
#         x=round(df_adstock_saturation['adstock'] * 100, 2),
#         y=df_adstock_saturation['canale'],
#         orientation='h'),
#         layout=dict(height=700, width=1000, xaxis_title='adstock geometric (%)', yaxis_title='canale',
#                     title='Geometric Adstock', legend_title='Legend')
#     )
#
#     fig.show()



import nevergrad as ng
from cassandra.model.nevergrad.utils import instrum_transformations_nevergrad, create_df_transformations_nevergrad, \
    choose_optimizer_algoritm, optimize_metric
from cassandra.model.utils import choose_model


def nevergrad_model(df, hyperparameters_dict, medias, others, target, use_intercept=True, use_adstock=True, use_saturation=True,
                    adstock_type='weibull', model_regression='linear', X_transformations_columns=[],
                    model_regression_log_log='linear', ridge_number=0, metric=None,
                    return_metric=True, size=0.2, positive=False, random_state=42, metric_return='mape_test',
                    force_coeffs=True, coeffs=[], intercept=0, optimizer_algoritm='TwoPointsDE', budget=1000,
                    num_workers=1):

    if metric is None:
        metric = ['rsq_train', 'rsq_test', 'nrmse_train', 'nrmse_test', 'mape_train', 'mape_test']

    value_trasf = instrum_transformations_nevergrad(hyperparameters_dict)

    metric_array = []
    df_transformations = df.copy()
    metrics_values = {}
    all_hyper = list(hyperparameters_dict.keys())

    def build_model(**all_hyper):
        df_transf = create_df_transformations_nevergrad(all_hyper, df, df_transformations, medias, use_adstock,
                                                        use_saturation, adstock_type)

        all_features = medias + others

        # CHECK
        if 'ridge_number' in all_hyper:
            nonlocal ridge_number
            ridge_number = all_hyper['ridge_number']

        if model_regression == 'logLog':
            all_features = [x for x in all_features if x not in X_transformations_columns]

        if force_coeffs:
            nonlocal coeffs
            nonlocal intercept
            if model_regression == 'logLog':
                all_features_coeffs = [value for key, value in all_hyper.items() if '_spend_' not in key if
                                       'ridge' not in key if key not in X_transformations_columns if
                                       'intercept' not in key]
                X_transformations_columns_coeffs = [value for key, value in all_hyper.items() if '_spend_' not in key if
                                                    'ridge' not in key if key not in all_features if
                                                    'intercept' not in key]
                if use_intercept:
                    intercept = [value for key, value in all_hyper.items() if 'intercept' in key]
                coeffs = X_transformations_columns_coeffs + all_features_coeffs
            else:
                all_coeffs = [value for key, value in all_hyper.items() if '_spend_' not in key if 'ridge' not in key]

                coeffs = all_coeffs[:-1]
                if use_intercept:
                    intercept = all_coeffs[-1]

        result, model, metrics_values = choose_model(df_transf, all_features, target, model_regression,
                                                     X_transformations_columns,
                                                     model_regression_log_log, ridge_number, metric,
                                                     return_metric, size, positive, random_state, force_coeffs,
                                                     coeffs, intercept)

        metric_to_optimize = optimize_metric(metric_return, metrics_values)
        metric_array.append(metric_to_optimize)

        return metric_to_optimize, result, model, metrics_values

    def build_model_result(**all_hyper):

        metric_to_optimize, result, model, metrics_values = build_model(**all_hyper)

        return metric_to_optimize

    instrum = ng.p.Instrumentation(**value_trasf)
    optimizer = choose_optimizer_algoritm(optimizer_algoritm, budget, num_workers, instrum)
    recommendation = optimizer.minimize(build_model_result)

    metric, result, model, metrics_values = build_model(**recommendation.value[1])
    return metrics_values, result, model, recommendation.value[1], metric_array



from cassandra.model.utils import choose_model


def nestedModel(df, X_columns, y_column, model_regression='linear', X_trasformations_columns=[],
                model_regression_log_log='linear', ridge_number=0, metric=None,
                return_metric=False, size=0.2, positive=False, random_state=42, force_coeffs=False, coeffs=[],
                intercept=0):
    if metric is None:
        metric = ['rsq_train', 'rsq_test', 'nrmse_train', 'nrmse_test', 'mape_train', 'mape_test']
    metrics_values = {}

    if return_metric:
        result, model, metrics_values = choose_model(df, X_columns, y_column, model_regression,
                                                     X_trasformations_columns,
                                                     model_regression_log_log, ridge_number, metric,
                                                     return_metric, size, positive, random_state, force_coeffs, coeffs,
                                                     intercept)
    else:
        result, model = choose_model(df, X_columns, y_column, model_regression, X_trasformations_columns,
                                     model_regression_log_log, ridge_number, metric,
                                     return_metric, size, positive, random_state, force_coeffs, coeffs, intercept)

    if metrics_values:
        return result, model, metrics_values
    else:
        return result, model


def mainNestedModel(sub_df, sub_X_columns, sub_y_column, X_columns, y_column, sub_model_regression='linear',
                    sub_X_trasformations_columns=[],
                    sub_model_regression_log_log='linear', sub_ridge_number=0, sub_metric=None,
                    return_sub_metric=False, sub_size=0.2, sub_positive=False, sub_random_state=42,
                    sub_force_coeffs=False, sub_coeffs=[], sub_intercept=0, model_regression='linear',
                    X_trasformations_columns=[], model_regression_log_log='linear', ridge_number=0,
                    metric=None, return_metric=False, size=0.2, positive=False, random_state=42,
                    force_coeffs=False, coeffs=[], intercept=0):
    if metric is None:
        sub_metric = ['rsq_train', 'rsq_test', 'nrmse_train', 'nrmse_test', 'mape_train', 'mape_test']
    if metric is None:
        metric = ['rsq_train', 'rsq_test', 'nrmse_train', 'nrmse_test', 'mape_train', 'mape_test']
    sub_metric_values = {}
    metrics_values = {}
    if return_sub_metric:
        sub_result, sub_model, sub_metric_values = nestedModel(sub_df, sub_X_columns, sub_y_column,
                                                               sub_model_regression, sub_X_trasformations_columns,
                                                               sub_model_regression_log_log, sub_ridge_number,
                                                               sub_metric, return_sub_metric, size=sub_size,
                                                               positive=sub_positive, random_state=sub_random_state,
                                                               force_coeffs=sub_force_coeffs, coeffs=sub_coeffs,
                                                               intercept=sub_intercept)
    else:
        sub_result, sub_model = nestedModel(sub_df, sub_X_columns, sub_y_column, sub_model_regression,
                                            sub_X_trasformations_columns,
                                            sub_model_regression_log_log, sub_ridge_number, sub_metric, size=sub_size,
                                            positive=sub_positive, random_state=sub_random_state,
                                            force_coeffs=sub_force_coeffs, coeffs=sub_coeffs,
                                            intercept=sub_intercept)

    df = sub_result.copy()

    df[sub_y_column] = sub_result['prediction'].copy()

    if return_metric:
        result, model, metrics_values = choose_model(df, X_columns, y_column, model_regression,
                                                     X_trasformations_columns,
                                                     model_regression_log_log, ridge_number, metric,
                                                     return_metric, size, positive, random_state, force_coeffs, coeffs,
                                                     intercept)
    else:
        result, model = choose_model(df, X_columns, y_column, model_regression, X_trasformations_columns,
                                     model_regression_log_log, ridge_number, metric,
                                     return_metric, size, positive, random_state, force_coeffs, coeffs, intercept)

    if metrics_values:
        return result, model, metrics_values, sub_result, sub_model, sub_metric_values
    else:
        return result, model, sub_result, sub_model



import os

def help_clean_and_format():
    path = os.path.join(os.path.dirname(__file__), "clean_and_format.txt")
    with open(path) as f:
        file_contents = f.read()
        print(file_contents)

def help_import():
    path = os.path.join(os.path.dirname(__file__), "import.txt")
    with open(path) as f:
        file_contents = f.read()
        print(file_contents)

def help_pivot():
    path = os.path.join(os.path.dirname(__file__), "import.txt")
    with open(path) as f:
        file_contents = f.read()
        print(file_contents)

def help_nevergrad():
    path = os.path.join(os.path.dirname(__file__), "import.txt")
    with open(path) as f:
        file_contents = f.read()
        print(file_contents)


import pandas as pd
from sklearn.preprocessing import OneHotEncoder
from cassandra.references.load_holidays import load_holidays


def create_dataset(path_csv, delimiter_csv=',', subset_columns=[], rename_columns={}):
    # Read data from a CSV file
    # @on_bad_lines - defines what to do in case a line is not properly formatted
    # @delimiter - specify the delimiter used in CSV
    df = pd.read_csv(path_csv, delimiter=delimiter_csv, on_bad_lines='skip')

    # Get a subset of variables, keep only the ones you need
    if subset_columns:
        df = df[subset_columns]

    # Rename variables in order to follow the same rules across all sources
    # It makes it easier to know what we are looking at
    # Hint: always keep the channel first in the name so it's easier to understand
    if rename_columns:
        df = df.rename(columns=rename_columns)

    return df

def create_data_columns(df, name_date_column, national_holidays_abbreviation = 'IT'):

    # Optional - If not already in datetime, cast your date variable as following
    df[name_date_column] = pd.to_datetime(df[name_date_column])

    # Create a column with number of day in the week
    df['day_week'] = pd.DatetimeIndex(df[name_date_column]).weekday

    df = df.reset_index(drop=True)

    # creating instance of one-hot-encoder
    # creating one column for every
    enc = OneHotEncoder(handle_unknown='ignore')
    enc_day_week = pd.DataFrame(enc.fit_transform(df[['day_week']]).toarray())
    df = df.join(enc_day_week)

    df.rename({
        0: 'mon',
        1: 'tue',
        2: 'wed',
        3: 'thu',
        4: 'fri',
        5: 'sat',
        6: 'sun'
    }, axis=1, inplace=True)

    # Create a column with week
    df['week'] = df[name_date_column].dt.strftime('%V')
    df = df.astype({"week": int})

    # Create a column with month
    df['month'] = pd.DatetimeIndex(df[name_date_column]).month
    enc_month = pd.DataFrame(enc.fit_transform(df[['month']]).toarray())
    df = df.join(enc_month)

    df.rename({
        0: 'jan',
        1: 'feb',
        2: 'mar',
        3: 'apr',
        4: 'may',
        5: 'jun',
        6: 'jul',
        7: 'aug',
        8: 'sep',
        9: 'oct',
        10: 'nov',
        11: 'dec'
    }, axis=1, inplace=True)

    # Create a column with year
    df['year'] = pd.DatetimeIndex(df[name_date_column]).year
    unique_year = df['year'].unique()
    enc_year = pd.DataFrame(enc.fit_transform(df[['year']]).toarray())
    df = df.join(enc_year)

    i = 0
    for x in unique_year:
        name = 'year_'+str(x)
        df.rename({
            i: name
        }, axis=1, inplace=True)
        i = i+1

    def data_weekly(row):
        return str(row['week']) + ' - ' + str(int(row['year']))

    # Create a column with number of the week and related year
    df['week_and_year'] = df.apply(lambda row: data_weekly(row), axis=1)

    for index, row in df.iterrows():
      if row['day_week'] == 5 or row['day_week'] == 6:
          df.at[index, 'weekend'] = 1
      else:
          df.at[index, 'weekend'] = 0



    csv_holiday = load_holidays()
    condition = (csv_holiday['country'] == national_holidays_abbreviation)
    national_holiday = csv_holiday.loc[condition, ['ds']]
    national_holiday['ds'] = pd.to_datetime(national_holiday['ds'])
    days_holiday = national_holiday.loc[:, 'ds'].values

    for index, row in df.iterrows():
      if row[name_date_column] in days_holiday:
          df.at[index, 'festivo'] = 1
      else:
          df.at[index, 'festivo'] = 0

    return df







